import { __assign, __decorate, __param } from 'tslib';
import { Inject, Injectable, EventEmitter, ElementRef, Renderer2, Input, Output, Directive, ContentChildren, Component, NgModule } from '@angular/core';
import { annotate } from 'rough-notation';
import { Subject } from 'rxjs';
import { filter, scan, tap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

var defaultColors = {
    highlight: '#FFF176',
    circle: '#0D47A1',
    box: '#4A148C',
    'strike-through': '#1B5E1F',
    underline: '#B71C1B',
    'crossed-off': '#F57F17',
};
var getDefaultTypeColor = function (type) {
    return defaultColors[type];
};
var defaultConfig = {
    type: 'highlight',
    color: getDefaultTypeColor('highlight'),
};

var RoughNotationService = /** @class */ (function () {
    function RoughNotationService(defaultConfig) {
        var _this = this;
        this.defaultConfig = defaultConfig;
        this.visibilitySubject = new Subject();
        this.inputConfig = {};
        this.visibilityState$ = this.visibilitySubject.asObservable().pipe(filter(function () { return !!_this.instance; }), scan(function (currentState) { return !currentState; }, false), tap(function (show) { return (show ? _this.instance.show() : _this.instance.hide()); }));
    }
    Object.defineProperty(RoughNotationService.prototype, "config", {
        /**
         * Generate config object from defaults and input overrides
         */
        get: function () {
            if (this.inputConfig.type && !this.inputConfig.color) {
                // When a type is specified without a color
                // Apply the default
                this.inputConfig.color = getDefaultTypeColor(this.inputConfig.type);
            }
            return __assign(__assign({}, this.defaultConfig), this.inputConfig);
        },
        enumerable: true,
        configurable: true
    });
    RoughNotationService.prototype.instantiate = function (element) {
        this.instance = annotate(element, this.config);
        return this.instance;
    };
    RoughNotationService.prototype.destroy = function () {
        this.instance.remove();
    };
    RoughNotationService.prototype.toggleVisibility = function () {
        this.visibilitySubject.next();
    };
    RoughNotationService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: ['DEFAULT_CONFIG',] }] }
    ]; };
    RoughNotationService = __decorate([
        Injectable(),
        __param(0, Inject('DEFAULT_CONFIG'))
    ], RoughNotationService);
    return RoughNotationService;
}());

var RoughNotationDirective = /** @class */ (function () {
    function RoughNotationDirective(hostElement, renderer, service) {
        var _this = this;
        this.hostElement = hostElement;
        this.renderer = renderer;
        this.service = service;
        this.isShowingChange = new EventEmitter();
        // Storing initial host element color
        this.initialTextColor = hostElement.nativeElement.style.color;
        this.showStateSubscription = this.service.visibilityState$.subscribe(function (state) {
            _this.isShowingChange.emit(state);
            _this._show = state;
            if (_this.annotatedTextColor) {
                // In case the user specified an annotation text color
                // We set or reset it according to the state
                var newColor = state ? _this.annotatedTextColor : _this.initialTextColor;
                _this.renderer.setStyle(_this.hostElement.nativeElement, 'color', newColor);
            }
        });
    }
    Object.defineProperty(RoughNotationDirective.prototype, "roughNotation", {
        set: function (value) {
            if (typeof value === 'string') {
                if (value) {
                    throw new Error('RoughAnnotationConfig object is expected as input');
                }
                this.service.inputConfig = {};
            }
            else {
                this.service.inputConfig = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RoughNotationDirective.prototype, "show", {
        get: function () {
            return this._show;
        },
        set: function (value) {
            if (value === false || this.instance) {
                this.setVisibility(value);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RoughNotationDirective.prototype, "instance", {
        get: function () {
            return this.service.instance;
        },
        enumerable: true,
        configurable: true
    });
    RoughNotationDirective.prototype.ngOnInit = function () {
        // Instantiate RoughNotation
        this.service.instantiate(this.hostElement.nativeElement);
        if (this._show !== false) {
            this.setVisibility(true);
        }
    };
    RoughNotationDirective.prototype.setVisibility = function (show) {
        if (this._show !== show) {
            this.toggle();
            this._show = show;
        }
    };
    RoughNotationDirective.prototype.toggle = function () {
        this.service.toggleVisibility();
    };
    RoughNotationDirective.prototype.ngOnDestroy = function () {
        var _a;
        (_a = this.showStateSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this.service.destroy();
    };
    RoughNotationDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: RoughNotationService }
    ]; };
    __decorate([
        Input('roughNotation')
    ], RoughNotationDirective.prototype, "roughNotation", null);
    __decorate([
        Input()
    ], RoughNotationDirective.prototype, "show", null);
    __decorate([
        Input('instance')
    ], RoughNotationDirective.prototype, "instance", null);
    __decorate([
        Input()
    ], RoughNotationDirective.prototype, "annotatedTextColor", void 0);
    __decorate([
        Output()
    ], RoughNotationDirective.prototype, "isShowingChange", void 0);
    RoughNotationDirective = __decorate([
        Directive({
            selector: '[roughNotation]',
            providers: [RoughNotationService],
            exportAs: 'roughNotation',
        })
    ], RoughNotationDirective);
    return RoughNotationDirective;
}());

var RoughNotationGroupComponent = /** @class */ (function () {
    function RoughNotationGroupComponent() {
        this._show = true;
    }
    Object.defineProperty(RoughNotationGroupComponent.prototype, "show", {
        set: function (value) {
            this.setVisibility(value);
        },
        enumerable: true,
        configurable: true
    });
    RoughNotationGroupComponent.prototype.ngAfterContentInit = function () {
        this.setVisibility(this._show);
    };
    RoughNotationGroupComponent.prototype.setVisibility = function (show) {
        if (this._show !== show) {
            this.toggleGroupMembers();
            this._show = show;
        }
    };
    RoughNotationGroupComponent.prototype.toggleGroupMembers = function () {
        this.annotations.forEach(function (annotation) { return annotation.toggle(); });
    };
    __decorate([
        Input()
    ], RoughNotationGroupComponent.prototype, "show", null);
    __decorate([
        ContentChildren(RoughNotationDirective, { descendants: true })
    ], RoughNotationGroupComponent.prototype, "annotations", void 0);
    RoughNotationGroupComponent = __decorate([
        Component({
            selector: 'rough-notation-group',
            template: "<ng-content></ng-content>\n",
            styles: [""]
        })
    ], RoughNotationGroupComponent);
    return RoughNotationGroupComponent;
}());

var ɵ0 = defaultConfig;
var RoughNotationModule = /** @class */ (function () {
    function RoughNotationModule() {
    }
    RoughNotationModule_1 = RoughNotationModule;
    RoughNotationModule.forRoot = function (defaultConfigOverride) {
        return {
            ngModule: RoughNotationModule_1,
            providers: [
                RoughNotationService,
                {
                    provide: 'DEFAULT_CONFIG',
                    useValue: defaultConfigOverride !== null && defaultConfigOverride !== void 0 ? defaultConfigOverride : defaultConfig,
                },
            ],
        };
    };
    var RoughNotationModule_1;
    RoughNotationModule = RoughNotationModule_1 = __decorate([
        NgModule({
            declarations: [RoughNotationDirective, RoughNotationGroupComponent],
            imports: [CommonModule],
            exports: [RoughNotationDirective, RoughNotationGroupComponent],
            providers: [
                RoughNotationService,
                {
                    provide: 'DEFAULT_CONFIG',
                    useValue: ɵ0,
                },
            ],
        })
    ], RoughNotationModule);
    return RoughNotationModule;
}());

/*
 * Public API Surface of ng-rough-notation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { RoughNotationDirective, RoughNotationGroupComponent, RoughNotationModule, RoughNotationService, ɵ0, getDefaultTypeColor as ɵa, defaultConfig as ɵb };
//# sourceMappingURL=ng-rough-notation.js.map
